// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: mapreferences.proto

package io.confluent.connect.protobuf.test;

public final class MapReferences {
  private MapReferences() {}
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface MapReferencesMessageOrBuilder extends
      // @@protoc_insertion_point(interface_extends:MapReferencesMessage)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> 
        getMap1List();
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getMap1(int index);
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    int getMap1Count();
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    java.util.List<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
        getMap1OrBuilderList();
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getMap1OrBuilder(
        int index);

    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> 
        getMap2List();
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getMap2(int index);
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    int getMap2Count();
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    java.util.List<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
        getMap2OrBuilderList();
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getMap2OrBuilder(
        int index);

    /**
     * <code>.AttributeFieldEntry notAMap1 = 3;</code>
     * @return Whether the notAMap1 field is set.
     */
    boolean hasNotAMap1();
    /**
     * <code>.AttributeFieldEntry notAMap1 = 3;</code>
     * @return The notAMap1.
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getNotAMap1();
    /**
     * <code>.AttributeFieldEntry notAMap1 = 3;</code>
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getNotAMap1OrBuilder();

    /**
     * <code>.AttributeFieldEntry notAMap2 = 4;</code>
     * @return Whether the notAMap2 field is set.
     */
    boolean hasNotAMap2();
    /**
     * <code>.AttributeFieldEntry notAMap2 = 4;</code>
     * @return The notAMap2.
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getNotAMap2();
    /**
     * <code>.AttributeFieldEntry notAMap2 = 4;</code>
     */
    io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getNotAMap2OrBuilder();
  }
  /**
   * Protobuf type {@code MapReferencesMessage}
   */
  public  static final class MapReferencesMessage extends
      com.google.protobuf.GeneratedMessageV3 implements
      // @@protoc_insertion_point(message_implements:MapReferencesMessage)
      MapReferencesMessageOrBuilder {
  private static final long serialVersionUID = 0L;
    // Use MapReferencesMessage.newBuilder() to construct.
    private MapReferencesMessage(com.google.protobuf.GeneratedMessageV3.Builder<?> builder) {
      super(builder);
    }
    private MapReferencesMessage() {
      map1_ = java.util.Collections.emptyList();
      map2_ = java.util.Collections.emptyList();
    }

    @java.lang.Override
    @SuppressWarnings({"unused"})
    protected java.lang.Object newInstance(
        UnusedPrivateParameter unused) {
      return new MapReferencesMessage();
    }

    @java.lang.Override
    public final com.google.protobuf.UnknownFieldSet
    getUnknownFields() {
      return this.unknownFields;
    }
    private MapReferencesMessage(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      this();
      if (extensionRegistry == null) {
        throw new java.lang.NullPointerException();
      }
      int mutable_bitField0_ = 0;
      com.google.protobuf.UnknownFieldSet.Builder unknownFields =
          com.google.protobuf.UnknownFieldSet.newBuilder();
      try {
        boolean done = false;
        while (!done) {
          int tag = input.readTag();
          switch (tag) {
            case 0:
              done = true;
              break;
            case 10: {
              if (!((mutable_bitField0_ & 0x00000001) != 0)) {
                map1_ = new java.util.ArrayList<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry>();
                mutable_bitField0_ |= 0x00000001;
              }
              map1_.add(
                  input.readMessage(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.parser(), extensionRegistry));
              break;
            }
            case 18: {
              if (!((mutable_bitField0_ & 0x00000002) != 0)) {
                map2_ = new java.util.ArrayList<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry>();
                mutable_bitField0_ |= 0x00000002;
              }
              map2_.add(
                  input.readMessage(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.parser(), extensionRegistry));
              break;
            }
            case 26: {
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder subBuilder = null;
              if (notAMap1_ != null) {
                subBuilder = notAMap1_.toBuilder();
              }
              notAMap1_ = input.readMessage(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.parser(), extensionRegistry);
              if (subBuilder != null) {
                subBuilder.mergeFrom(notAMap1_);
                notAMap1_ = subBuilder.buildPartial();
              }

              break;
            }
            case 34: {
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder subBuilder = null;
              if (notAMap2_ != null) {
                subBuilder = notAMap2_.toBuilder();
              }
              notAMap2_ = input.readMessage(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.parser(), extensionRegistry);
              if (subBuilder != null) {
                subBuilder.mergeFrom(notAMap2_);
                notAMap2_ = subBuilder.buildPartial();
              }

              break;
            }
            default: {
              if (!parseUnknownField(
                  input, unknownFields, extensionRegistry, tag)) {
                done = true;
              }
              break;
            }
          }
        }
      } catch (com.google.protobuf.InvalidProtocolBufferException e) {
        throw e.setUnfinishedMessage(this);
      } catch (java.io.IOException e) {
        throw new com.google.protobuf.InvalidProtocolBufferException(
            e).setUnfinishedMessage(this);
      } finally {
        if (((mutable_bitField0_ & 0x00000001) != 0)) {
          map1_ = java.util.Collections.unmodifiableList(map1_);
        }
        if (((mutable_bitField0_ & 0x00000002) != 0)) {
          map2_ = java.util.Collections.unmodifiableList(map2_);
        }
        this.unknownFields = unknownFields.build();
        makeExtensionsImmutable();
      }
    }
    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return io.confluent.connect.protobuf.test.MapReferences.internal_static_MapReferencesMessage_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return io.confluent.connect.protobuf.test.MapReferences.internal_static_MapReferencesMessage_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.class, io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.Builder.class);
    }

    public static final int MAP1_FIELD_NUMBER = 1;
    private java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> map1_;
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    public java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> getMap1List() {
      return map1_;
    }
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    public java.util.List<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
        getMap1OrBuilderList() {
      return map1_;
    }
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    public int getMap1Count() {
      return map1_.size();
    }
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getMap1(int index) {
      return map1_.get(index);
    }
    /**
     * <code>repeated .AttributeFieldEntry map1 = 1;</code>
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getMap1OrBuilder(
        int index) {
      return map1_.get(index);
    }

    public static final int MAP2_FIELD_NUMBER = 2;
    private java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> map2_;
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    public java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> getMap2List() {
      return map2_;
    }
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    public java.util.List<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
        getMap2OrBuilderList() {
      return map2_;
    }
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    public int getMap2Count() {
      return map2_.size();
    }
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getMap2(int index) {
      return map2_.get(index);
    }
    /**
     * <code>repeated .AttributeFieldEntry map2 = 2;</code>
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getMap2OrBuilder(
        int index) {
      return map2_.get(index);
    }

    public static final int NOTAMAP1_FIELD_NUMBER = 3;
    private io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry notAMap1_;
    /**
     * <code>.AttributeFieldEntry notAMap1 = 3;</code>
     * @return Whether the notAMap1 field is set.
     */
    public boolean hasNotAMap1() {
      return notAMap1_ != null;
    }
    /**
     * <code>.AttributeFieldEntry notAMap1 = 3;</code>
     * @return The notAMap1.
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getNotAMap1() {
      return notAMap1_ == null ? io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance() : notAMap1_;
    }
    /**
     * <code>.AttributeFieldEntry notAMap1 = 3;</code>
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getNotAMap1OrBuilder() {
      return getNotAMap1();
    }

    public static final int NOTAMAP2_FIELD_NUMBER = 4;
    private io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry notAMap2_;
    /**
     * <code>.AttributeFieldEntry notAMap2 = 4;</code>
     * @return Whether the notAMap2 field is set.
     */
    public boolean hasNotAMap2() {
      return notAMap2_ != null;
    }
    /**
     * <code>.AttributeFieldEntry notAMap2 = 4;</code>
     * @return The notAMap2.
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getNotAMap2() {
      return notAMap2_ == null ? io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance() : notAMap2_;
    }
    /**
     * <code>.AttributeFieldEntry notAMap2 = 4;</code>
     */
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getNotAMap2OrBuilder() {
      return getNotAMap2();
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      for (int i = 0; i < map1_.size(); i++) {
        output.writeMessage(1, map1_.get(i));
      }
      for (int i = 0; i < map2_.size(); i++) {
        output.writeMessage(2, map2_.get(i));
      }
      if (notAMap1_ != null) {
        output.writeMessage(3, getNotAMap1());
      }
      if (notAMap2_ != null) {
        output.writeMessage(4, getNotAMap2());
      }
      unknownFields.writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      for (int i = 0; i < map1_.size(); i++) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(1, map1_.get(i));
      }
      for (int i = 0; i < map2_.size(); i++) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(2, map2_.get(i));
      }
      if (notAMap1_ != null) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(3, getNotAMap1());
      }
      if (notAMap2_ != null) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(4, getNotAMap2());
      }
      size += unknownFields.getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage)) {
        return super.equals(obj);
      }
      io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage other = (io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage) obj;

      if (!getMap1List()
          .equals(other.getMap1List())) return false;
      if (!getMap2List()
          .equals(other.getMap2List())) return false;
      if (hasNotAMap1() != other.hasNotAMap1()) return false;
      if (hasNotAMap1()) {
        if (!getNotAMap1()
            .equals(other.getNotAMap1())) return false;
      }
      if (hasNotAMap2() != other.hasNotAMap2()) return false;
      if (hasNotAMap2()) {
        if (!getNotAMap2()
            .equals(other.getNotAMap2())) return false;
      }
      if (!unknownFields.equals(other.unknownFields)) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (getMap1Count() > 0) {
        hash = (37 * hash) + MAP1_FIELD_NUMBER;
        hash = (53 * hash) + getMap1List().hashCode();
      }
      if (getMap2Count() > 0) {
        hash = (37 * hash) + MAP2_FIELD_NUMBER;
        hash = (53 * hash) + getMap2List().hashCode();
      }
      if (hasNotAMap1()) {
        hash = (37 * hash) + NOTAMAP1_FIELD_NUMBER;
        hash = (53 * hash) + getNotAMap1().hashCode();
      }
      if (hasNotAMap2()) {
        hash = (37 * hash) + NOTAMAP2_FIELD_NUMBER;
        hash = (53 * hash) + getNotAMap2().hashCode();
      }
      hash = (29 * hash) + unknownFields.hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseDelimitedWithIOException(PARSER, input);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessageV3.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code MapReferencesMessage}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessageV3.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:MapReferencesMessage)
        io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessageOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return io.confluent.connect.protobuf.test.MapReferences.internal_static_MapReferencesMessage_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return io.confluent.connect.protobuf.test.MapReferences.internal_static_MapReferencesMessage_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.class, io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.Builder.class);
      }

      // Construct using io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.newBuilder()
      private Builder() {
        maybeForceBuilderInitialization();
      }

      private Builder(
          com.google.protobuf.GeneratedMessageV3.BuilderParent parent) {
        super(parent);
        maybeForceBuilderInitialization();
      }
      private void maybeForceBuilderInitialization() {
        if (com.google.protobuf.GeneratedMessageV3
                .alwaysUseFieldBuilders) {
          getMap1FieldBuilder();
          getMap2FieldBuilder();
        }
      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        if (map1Builder_ == null) {
          map1_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00000001);
        } else {
          map1Builder_.clear();
        }
        if (map2Builder_ == null) {
          map2_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00000002);
        } else {
          map2Builder_.clear();
        }
        if (notAMap1Builder_ == null) {
          notAMap1_ = null;
        } else {
          notAMap1_ = null;
          notAMap1Builder_ = null;
        }
        if (notAMap2Builder_ == null) {
          notAMap2_ = null;
        } else {
          notAMap2_ = null;
          notAMap2Builder_ = null;
        }
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return io.confluent.connect.protobuf.test.MapReferences.internal_static_MapReferencesMessage_descriptor;
      }

      @java.lang.Override
      public io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage getDefaultInstanceForType() {
        return io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.getDefaultInstance();
      }

      @java.lang.Override
      public io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage build() {
        io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage buildPartial() {
        io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage result = new io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage(this);
        int from_bitField0_ = bitField0_;
        if (map1Builder_ == null) {
          if (((bitField0_ & 0x00000001) != 0)) {
            map1_ = java.util.Collections.unmodifiableList(map1_);
            bitField0_ = (bitField0_ & ~0x00000001);
          }
          result.map1_ = map1_;
        } else {
          result.map1_ = map1Builder_.build();
        }
        if (map2Builder_ == null) {
          if (((bitField0_ & 0x00000002) != 0)) {
            map2_ = java.util.Collections.unmodifiableList(map2_);
            bitField0_ = (bitField0_ & ~0x00000002);
          }
          result.map2_ = map2_;
        } else {
          result.map2_ = map2Builder_.build();
        }
        if (notAMap1Builder_ == null) {
          result.notAMap1_ = notAMap1_;
        } else {
          result.notAMap1_ = notAMap1Builder_.build();
        }
        if (notAMap2Builder_ == null) {
          result.notAMap2_ = notAMap2_;
        } else {
          result.notAMap2_ = notAMap2Builder_.build();
        }
        onBuilt();
        return result;
      }

      @java.lang.Override
      public Builder clone() {
        return super.clone();
      }
      @java.lang.Override
      public Builder setField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          java.lang.Object value) {
        return super.setField(field, value);
      }
      @java.lang.Override
      public Builder clearField(
          com.google.protobuf.Descriptors.FieldDescriptor field) {
        return super.clearField(field);
      }
      @java.lang.Override
      public Builder clearOneof(
          com.google.protobuf.Descriptors.OneofDescriptor oneof) {
        return super.clearOneof(oneof);
      }
      @java.lang.Override
      public Builder setRepeatedField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          int index, java.lang.Object value) {
        return super.setRepeatedField(field, index, value);
      }
      @java.lang.Override
      public Builder addRepeatedField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          java.lang.Object value) {
        return super.addRepeatedField(field, value);
      }
      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage) {
          return mergeFrom((io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage other) {
        if (other == io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage.getDefaultInstance()) return this;
        if (map1Builder_ == null) {
          if (!other.map1_.isEmpty()) {
            if (map1_.isEmpty()) {
              map1_ = other.map1_;
              bitField0_ = (bitField0_ & ~0x00000001);
            } else {
              ensureMap1IsMutable();
              map1_.addAll(other.map1_);
            }
            onChanged();
          }
        } else {
          if (!other.map1_.isEmpty()) {
            if (map1Builder_.isEmpty()) {
              map1Builder_.dispose();
              map1Builder_ = null;
              map1_ = other.map1_;
              bitField0_ = (bitField0_ & ~0x00000001);
              map1Builder_ = 
                com.google.protobuf.GeneratedMessageV3.alwaysUseFieldBuilders ?
                   getMap1FieldBuilder() : null;
            } else {
              map1Builder_.addAllMessages(other.map1_);
            }
          }
        }
        if (map2Builder_ == null) {
          if (!other.map2_.isEmpty()) {
            if (map2_.isEmpty()) {
              map2_ = other.map2_;
              bitField0_ = (bitField0_ & ~0x00000002);
            } else {
              ensureMap2IsMutable();
              map2_.addAll(other.map2_);
            }
            onChanged();
          }
        } else {
          if (!other.map2_.isEmpty()) {
            if (map2Builder_.isEmpty()) {
              map2Builder_.dispose();
              map2Builder_ = null;
              map2_ = other.map2_;
              bitField0_ = (bitField0_ & ~0x00000002);
              map2Builder_ = 
                com.google.protobuf.GeneratedMessageV3.alwaysUseFieldBuilders ?
                   getMap2FieldBuilder() : null;
            } else {
              map2Builder_.addAllMessages(other.map2_);
            }
          }
        }
        if (other.hasNotAMap1()) {
          mergeNotAMap1(other.getNotAMap1());
        }
        if (other.hasNotAMap2()) {
          mergeNotAMap2(other.getNotAMap2());
        }
        this.mergeUnknownFields(other.unknownFields);
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage parsedMessage = null;
        try {
          parsedMessage = PARSER.parsePartialFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          parsedMessage = (io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage) e.getUnfinishedMessage();
          throw e.unwrapIOException();
        } finally {
          if (parsedMessage != null) {
            mergeFrom(parsedMessage);
          }
        }
        return this;
      }
      private int bitField0_;

      private java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> map1_ =
        java.util.Collections.emptyList();
      private void ensureMap1IsMutable() {
        if (!((bitField0_ & 0x00000001) != 0)) {
          map1_ = new java.util.ArrayList<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry>(map1_);
          bitField0_ |= 0x00000001;
         }
      }

      private com.google.protobuf.RepeatedFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> map1Builder_;

      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> getMap1List() {
        if (map1Builder_ == null) {
          return java.util.Collections.unmodifiableList(map1_);
        } else {
          return map1Builder_.getMessageList();
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public int getMap1Count() {
        if (map1Builder_ == null) {
          return map1_.size();
        } else {
          return map1Builder_.getCount();
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getMap1(int index) {
        if (map1Builder_ == null) {
          return map1_.get(index);
        } else {
          return map1Builder_.getMessage(index);
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder setMap1(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (map1Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureMap1IsMutable();
          map1_.set(index, value);
          onChanged();
        } else {
          map1Builder_.setMessage(index, value);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder setMap1(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (map1Builder_ == null) {
          ensureMap1IsMutable();
          map1_.set(index, builderForValue.build());
          onChanged();
        } else {
          map1Builder_.setMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder addMap1(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (map1Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureMap1IsMutable();
          map1_.add(value);
          onChanged();
        } else {
          map1Builder_.addMessage(value);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder addMap1(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (map1Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureMap1IsMutable();
          map1_.add(index, value);
          onChanged();
        } else {
          map1Builder_.addMessage(index, value);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder addMap1(
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (map1Builder_ == null) {
          ensureMap1IsMutable();
          map1_.add(builderForValue.build());
          onChanged();
        } else {
          map1Builder_.addMessage(builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder addMap1(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (map1Builder_ == null) {
          ensureMap1IsMutable();
          map1_.add(index, builderForValue.build());
          onChanged();
        } else {
          map1Builder_.addMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder addAllMap1(
          java.lang.Iterable<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> values) {
        if (map1Builder_ == null) {
          ensureMap1IsMutable();
          com.google.protobuf.AbstractMessageLite.Builder.addAll(
              values, map1_);
          onChanged();
        } else {
          map1Builder_.addAllMessages(values);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder clearMap1() {
        if (map1Builder_ == null) {
          map1_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00000001);
          onChanged();
        } else {
          map1Builder_.clear();
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public Builder removeMap1(int index) {
        if (map1Builder_ == null) {
          ensureMap1IsMutable();
          map1_.remove(index);
          onChanged();
        } else {
          map1Builder_.remove(index);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder getMap1Builder(
          int index) {
        return getMap1FieldBuilder().getBuilder(index);
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getMap1OrBuilder(
          int index) {
        if (map1Builder_ == null) {
          return map1_.get(index);  } else {
          return map1Builder_.getMessageOrBuilder(index);
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public java.util.List<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
           getMap1OrBuilderList() {
        if (map1Builder_ != null) {
          return map1Builder_.getMessageOrBuilderList();
        } else {
          return java.util.Collections.unmodifiableList(map1_);
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder addMap1Builder() {
        return getMap1FieldBuilder().addBuilder(
            io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance());
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder addMap1Builder(
          int index) {
        return getMap1FieldBuilder().addBuilder(
            index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance());
      }
      /**
       * <code>repeated .AttributeFieldEntry map1 = 1;</code>
       */
      public java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder> 
           getMap1BuilderList() {
        return getMap1FieldBuilder().getBuilderList();
      }
      private com.google.protobuf.RepeatedFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
          getMap1FieldBuilder() {
        if (map1Builder_ == null) {
          map1Builder_ = new com.google.protobuf.RepeatedFieldBuilderV3<
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder>(
                  map1_,
                  ((bitField0_ & 0x00000001) != 0),
                  getParentForChildren(),
                  isClean());
          map1_ = null;
        }
        return map1Builder_;
      }

      private java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> map2_ =
        java.util.Collections.emptyList();
      private void ensureMap2IsMutable() {
        if (!((bitField0_ & 0x00000002) != 0)) {
          map2_ = new java.util.ArrayList<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry>(map2_);
          bitField0_ |= 0x00000002;
         }
      }

      private com.google.protobuf.RepeatedFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> map2Builder_;

      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> getMap2List() {
        if (map2Builder_ == null) {
          return java.util.Collections.unmodifiableList(map2_);
        } else {
          return map2Builder_.getMessageList();
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public int getMap2Count() {
        if (map2Builder_ == null) {
          return map2_.size();
        } else {
          return map2Builder_.getCount();
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getMap2(int index) {
        if (map2Builder_ == null) {
          return map2_.get(index);
        } else {
          return map2Builder_.getMessage(index);
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder setMap2(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (map2Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureMap2IsMutable();
          map2_.set(index, value);
          onChanged();
        } else {
          map2Builder_.setMessage(index, value);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder setMap2(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (map2Builder_ == null) {
          ensureMap2IsMutable();
          map2_.set(index, builderForValue.build());
          onChanged();
        } else {
          map2Builder_.setMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder addMap2(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (map2Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureMap2IsMutable();
          map2_.add(value);
          onChanged();
        } else {
          map2Builder_.addMessage(value);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder addMap2(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (map2Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureMap2IsMutable();
          map2_.add(index, value);
          onChanged();
        } else {
          map2Builder_.addMessage(index, value);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder addMap2(
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (map2Builder_ == null) {
          ensureMap2IsMutable();
          map2_.add(builderForValue.build());
          onChanged();
        } else {
          map2Builder_.addMessage(builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder addMap2(
          int index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (map2Builder_ == null) {
          ensureMap2IsMutable();
          map2_.add(index, builderForValue.build());
          onChanged();
        } else {
          map2Builder_.addMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder addAllMap2(
          java.lang.Iterable<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry> values) {
        if (map2Builder_ == null) {
          ensureMap2IsMutable();
          com.google.protobuf.AbstractMessageLite.Builder.addAll(
              values, map2_);
          onChanged();
        } else {
          map2Builder_.addAllMessages(values);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder clearMap2() {
        if (map2Builder_ == null) {
          map2_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00000002);
          onChanged();
        } else {
          map2Builder_.clear();
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public Builder removeMap2(int index) {
        if (map2Builder_ == null) {
          ensureMap2IsMutable();
          map2_.remove(index);
          onChanged();
        } else {
          map2Builder_.remove(index);
        }
        return this;
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder getMap2Builder(
          int index) {
        return getMap2FieldBuilder().getBuilder(index);
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getMap2OrBuilder(
          int index) {
        if (map2Builder_ == null) {
          return map2_.get(index);  } else {
          return map2Builder_.getMessageOrBuilder(index);
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public java.util.List<? extends io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
           getMap2OrBuilderList() {
        if (map2Builder_ != null) {
          return map2Builder_.getMessageOrBuilderList();
        } else {
          return java.util.Collections.unmodifiableList(map2_);
        }
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder addMap2Builder() {
        return getMap2FieldBuilder().addBuilder(
            io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance());
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder addMap2Builder(
          int index) {
        return getMap2FieldBuilder().addBuilder(
            index, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance());
      }
      /**
       * <code>repeated .AttributeFieldEntry map2 = 2;</code>
       */
      public java.util.List<io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder> 
           getMap2BuilderList() {
        return getMap2FieldBuilder().getBuilderList();
      }
      private com.google.protobuf.RepeatedFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
          getMap2FieldBuilder() {
        if (map2Builder_ == null) {
          map2Builder_ = new com.google.protobuf.RepeatedFieldBuilderV3<
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder>(
                  map2_,
                  ((bitField0_ & 0x00000002) != 0),
                  getParentForChildren(),
                  isClean());
          map2_ = null;
        }
        return map2Builder_;
      }

      private io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry notAMap1_;
      private com.google.protobuf.SingleFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> notAMap1Builder_;
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       * @return Whether the notAMap1 field is set.
       */
      public boolean hasNotAMap1() {
        return notAMap1Builder_ != null || notAMap1_ != null;
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       * @return The notAMap1.
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getNotAMap1() {
        if (notAMap1Builder_ == null) {
          return notAMap1_ == null ? io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance() : notAMap1_;
        } else {
          return notAMap1Builder_.getMessage();
        }
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      public Builder setNotAMap1(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (notAMap1Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          notAMap1_ = value;
          onChanged();
        } else {
          notAMap1Builder_.setMessage(value);
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      public Builder setNotAMap1(
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (notAMap1Builder_ == null) {
          notAMap1_ = builderForValue.build();
          onChanged();
        } else {
          notAMap1Builder_.setMessage(builderForValue.build());
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      public Builder mergeNotAMap1(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (notAMap1Builder_ == null) {
          if (notAMap1_ != null) {
            notAMap1_ =
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.newBuilder(notAMap1_).mergeFrom(value).buildPartial();
          } else {
            notAMap1_ = value;
          }
          onChanged();
        } else {
          notAMap1Builder_.mergeFrom(value);
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      public Builder clearNotAMap1() {
        if (notAMap1Builder_ == null) {
          notAMap1_ = null;
          onChanged();
        } else {
          notAMap1_ = null;
          notAMap1Builder_ = null;
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder getNotAMap1Builder() {
        
        onChanged();
        return getNotAMap1FieldBuilder().getBuilder();
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getNotAMap1OrBuilder() {
        if (notAMap1Builder_ != null) {
          return notAMap1Builder_.getMessageOrBuilder();
        } else {
          return notAMap1_ == null ?
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance() : notAMap1_;
        }
      }
      /**
       * <code>.AttributeFieldEntry notAMap1 = 3;</code>
       */
      private com.google.protobuf.SingleFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
          getNotAMap1FieldBuilder() {
        if (notAMap1Builder_ == null) {
          notAMap1Builder_ = new com.google.protobuf.SingleFieldBuilderV3<
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder>(
                  getNotAMap1(),
                  getParentForChildren(),
                  isClean());
          notAMap1_ = null;
        }
        return notAMap1Builder_;
      }

      private io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry notAMap2_;
      private com.google.protobuf.SingleFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> notAMap2Builder_;
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       * @return Whether the notAMap2 field is set.
       */
      public boolean hasNotAMap2() {
        return notAMap2Builder_ != null || notAMap2_ != null;
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       * @return The notAMap2.
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getNotAMap2() {
        if (notAMap2Builder_ == null) {
          return notAMap2_ == null ? io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance() : notAMap2_;
        } else {
          return notAMap2Builder_.getMessage();
        }
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      public Builder setNotAMap2(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (notAMap2Builder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          notAMap2_ = value;
          onChanged();
        } else {
          notAMap2Builder_.setMessage(value);
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      public Builder setNotAMap2(
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder builderForValue) {
        if (notAMap2Builder_ == null) {
          notAMap2_ = builderForValue.build();
          onChanged();
        } else {
          notAMap2Builder_.setMessage(builderForValue.build());
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      public Builder mergeNotAMap2(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry value) {
        if (notAMap2Builder_ == null) {
          if (notAMap2_ != null) {
            notAMap2_ =
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.newBuilder(notAMap2_).mergeFrom(value).buildPartial();
          } else {
            notAMap2_ = value;
          }
          onChanged();
        } else {
          notAMap2Builder_.mergeFrom(value);
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      public Builder clearNotAMap2() {
        if (notAMap2Builder_ == null) {
          notAMap2_ = null;
          onChanged();
        } else {
          notAMap2_ = null;
          notAMap2Builder_ = null;
        }

        return this;
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder getNotAMap2Builder() {
        
        onChanged();
        return getNotAMap2FieldBuilder().getBuilder();
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder getNotAMap2OrBuilder() {
        if (notAMap2Builder_ != null) {
          return notAMap2Builder_.getMessageOrBuilder();
        } else {
          return notAMap2_ == null ?
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance() : notAMap2_;
        }
      }
      /**
       * <code>.AttributeFieldEntry notAMap2 = 4;</code>
       */
      private com.google.protobuf.SingleFieldBuilderV3<
          io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder> 
          getNotAMap2FieldBuilder() {
        if (notAMap2Builder_ == null) {
          notAMap2Builder_ = new com.google.protobuf.SingleFieldBuilderV3<
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder>(
                  getNotAMap2(),
                  getParentForChildren(),
                  isClean());
          notAMap2_ = null;
        }
        return notAMap2Builder_;
      }
      @java.lang.Override
      public final Builder setUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return super.setUnknownFields(unknownFields);
      }

      @java.lang.Override
      public final Builder mergeUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return super.mergeUnknownFields(unknownFields);
      }


      // @@protoc_insertion_point(builder_scope:MapReferencesMessage)
    }

    // @@protoc_insertion_point(class_scope:MapReferencesMessage)
    private static final io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage();
    }

    public static io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<MapReferencesMessage>
        PARSER = new com.google.protobuf.AbstractParser<MapReferencesMessage>() {
      @java.lang.Override
      public MapReferencesMessage parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        return new MapReferencesMessage(input, extensionRegistry);
      }
    };

    public static com.google.protobuf.Parser<MapReferencesMessage> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<MapReferencesMessage> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public io.confluent.connect.protobuf.test.MapReferences.MapReferencesMessage getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  public interface AttributeFieldEntryOrBuilder extends
      // @@protoc_insertion_point(interface_extends:AttributeFieldEntry)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>string key = 1;</code>
     * @return The key.
     */
    java.lang.String getKey();
    /**
     * <code>string key = 1;</code>
     * @return The bytes for key.
     */
    com.google.protobuf.ByteString
        getKeyBytes();

    /**
     * <code>string value = 2;</code>
     * @return The value.
     */
    java.lang.String getValue();
    /**
     * <code>string value = 2;</code>
     * @return The bytes for value.
     */
    com.google.protobuf.ByteString
        getValueBytes();
  }
  /**
   * Protobuf type {@code AttributeFieldEntry}
   */
  public  static final class AttributeFieldEntry extends
      com.google.protobuf.GeneratedMessageV3 implements
      // @@protoc_insertion_point(message_implements:AttributeFieldEntry)
      AttributeFieldEntryOrBuilder {
  private static final long serialVersionUID = 0L;
    // Use AttributeFieldEntry.newBuilder() to construct.
    private AttributeFieldEntry(com.google.protobuf.GeneratedMessageV3.Builder<?> builder) {
      super(builder);
    }
    private AttributeFieldEntry() {
      key_ = "";
      value_ = "";
    }

    @java.lang.Override
    @SuppressWarnings({"unused"})
    protected java.lang.Object newInstance(
        UnusedPrivateParameter unused) {
      return new AttributeFieldEntry();
    }

    @java.lang.Override
    public final com.google.protobuf.UnknownFieldSet
    getUnknownFields() {
      return this.unknownFields;
    }
    private AttributeFieldEntry(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      this();
      if (extensionRegistry == null) {
        throw new java.lang.NullPointerException();
      }
      com.google.protobuf.UnknownFieldSet.Builder unknownFields =
          com.google.protobuf.UnknownFieldSet.newBuilder();
      try {
        boolean done = false;
        while (!done) {
          int tag = input.readTag();
          switch (tag) {
            case 0:
              done = true;
              break;
            case 10: {
              java.lang.String s = input.readStringRequireUtf8();

              key_ = s;
              break;
            }
            case 18: {
              java.lang.String s = input.readStringRequireUtf8();

              value_ = s;
              break;
            }
            default: {
              if (!parseUnknownField(
                  input, unknownFields, extensionRegistry, tag)) {
                done = true;
              }
              break;
            }
          }
        }
      } catch (com.google.protobuf.InvalidProtocolBufferException e) {
        throw e.setUnfinishedMessage(this);
      } catch (java.io.IOException e) {
        throw new com.google.protobuf.InvalidProtocolBufferException(
            e).setUnfinishedMessage(this);
      } finally {
        this.unknownFields = unknownFields.build();
        makeExtensionsImmutable();
      }
    }
    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return io.confluent.connect.protobuf.test.MapReferences.internal_static_AttributeFieldEntry_descriptor;
    }

    @java.lang.Override
    protected com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return io.confluent.connect.protobuf.test.MapReferences.internal_static_AttributeFieldEntry_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.class, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder.class);
    }

    public static final int KEY_FIELD_NUMBER = 1;
    private volatile java.lang.Object key_;
    /**
     * <code>string key = 1;</code>
     * @return The key.
     */
    public java.lang.String getKey() {
      java.lang.Object ref = key_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        key_ = s;
        return s;
      }
    }
    /**
     * <code>string key = 1;</code>
     * @return The bytes for key.
     */
    public com.google.protobuf.ByteString
        getKeyBytes() {
      java.lang.Object ref = key_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        key_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int VALUE_FIELD_NUMBER = 2;
    private volatile java.lang.Object value_;
    /**
     * <code>string value = 2;</code>
     * @return The value.
     */
    public java.lang.String getValue() {
      java.lang.Object ref = value_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        value_ = s;
        return s;
      }
    }
    /**
     * <code>string value = 2;</code>
     * @return The bytes for value.
     */
    public com.google.protobuf.ByteString
        getValueBytes() {
      java.lang.Object ref = value_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        value_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (!getKeyBytes().isEmpty()) {
        com.google.protobuf.GeneratedMessageV3.writeString(output, 1, key_);
      }
      if (!getValueBytes().isEmpty()) {
        com.google.protobuf.GeneratedMessageV3.writeString(output, 2, value_);
      }
      unknownFields.writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (!getKeyBytes().isEmpty()) {
        size += com.google.protobuf.GeneratedMessageV3.computeStringSize(1, key_);
      }
      if (!getValueBytes().isEmpty()) {
        size += com.google.protobuf.GeneratedMessageV3.computeStringSize(2, value_);
      }
      size += unknownFields.getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry)) {
        return super.equals(obj);
      }
      io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry other = (io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry) obj;

      if (!getKey()
          .equals(other.getKey())) return false;
      if (!getValue()
          .equals(other.getValue())) return false;
      if (!unknownFields.equals(other.unknownFields)) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      hash = (37 * hash) + KEY_FIELD_NUMBER;
      hash = (53 * hash) + getKey().hashCode();
      hash = (37 * hash) + VALUE_FIELD_NUMBER;
      hash = (53 * hash) + getValue().hashCode();
      hash = (29 * hash) + unknownFields.hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseDelimitedWithIOException(PARSER, input);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input);
    }
    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessageV3.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code AttributeFieldEntry}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessageV3.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:AttributeFieldEntry)
        io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntryOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return io.confluent.connect.protobuf.test.MapReferences.internal_static_AttributeFieldEntry_descriptor;
      }

      @java.lang.Override
      protected com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return io.confluent.connect.protobuf.test.MapReferences.internal_static_AttributeFieldEntry_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.class, io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.Builder.class);
      }

      // Construct using io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.newBuilder()
      private Builder() {
        maybeForceBuilderInitialization();
      }

      private Builder(
          com.google.protobuf.GeneratedMessageV3.BuilderParent parent) {
        super(parent);
        maybeForceBuilderInitialization();
      }
      private void maybeForceBuilderInitialization() {
        if (com.google.protobuf.GeneratedMessageV3
                .alwaysUseFieldBuilders) {
        }
      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        key_ = "";

        value_ = "";

        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return io.confluent.connect.protobuf.test.MapReferences.internal_static_AttributeFieldEntry_descriptor;
      }

      @java.lang.Override
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getDefaultInstanceForType() {
        return io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance();
      }

      @java.lang.Override
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry build() {
        io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry buildPartial() {
        io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry result = new io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry(this);
        result.key_ = key_;
        result.value_ = value_;
        onBuilt();
        return result;
      }

      @java.lang.Override
      public Builder clone() {
        return super.clone();
      }
      @java.lang.Override
      public Builder setField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          java.lang.Object value) {
        return super.setField(field, value);
      }
      @java.lang.Override
      public Builder clearField(
          com.google.protobuf.Descriptors.FieldDescriptor field) {
        return super.clearField(field);
      }
      @java.lang.Override
      public Builder clearOneof(
          com.google.protobuf.Descriptors.OneofDescriptor oneof) {
        return super.clearOneof(oneof);
      }
      @java.lang.Override
      public Builder setRepeatedField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          int index, java.lang.Object value) {
        return super.setRepeatedField(field, index, value);
      }
      @java.lang.Override
      public Builder addRepeatedField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          java.lang.Object value) {
        return super.addRepeatedField(field, value);
      }
      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry) {
          return mergeFrom((io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry other) {
        if (other == io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry.getDefaultInstance()) return this;
        if (!other.getKey().isEmpty()) {
          key_ = other.key_;
          onChanged();
        }
        if (!other.getValue().isEmpty()) {
          value_ = other.value_;
          onChanged();
        }
        this.mergeUnknownFields(other.unknownFields);
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry parsedMessage = null;
        try {
          parsedMessage = PARSER.parsePartialFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          parsedMessage = (io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry) e.getUnfinishedMessage();
          throw e.unwrapIOException();
        } finally {
          if (parsedMessage != null) {
            mergeFrom(parsedMessage);
          }
        }
        return this;
      }

      private java.lang.Object key_ = "";
      /**
       * <code>string key = 1;</code>
       * @return The key.
       */
      public java.lang.String getKey() {
        java.lang.Object ref = key_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          key_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>string key = 1;</code>
       * @return The bytes for key.
       */
      public com.google.protobuf.ByteString
          getKeyBytes() {
        java.lang.Object ref = key_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          key_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>string key = 1;</code>
       * @param value The key to set.
       * @return This builder for chaining.
       */
      public Builder setKey(
          java.lang.String value) {
        if (value == null) {
    throw new NullPointerException();
  }
  
        key_ = value;
        onChanged();
        return this;
      }
      /**
       * <code>string key = 1;</code>
       * @return This builder for chaining.
       */
      public Builder clearKey() {
        
        key_ = getDefaultInstance().getKey();
        onChanged();
        return this;
      }
      /**
       * <code>string key = 1;</code>
       * @param value The bytes for key to set.
       * @return This builder for chaining.
       */
      public Builder setKeyBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) {
    throw new NullPointerException();
  }
  checkByteStringIsUtf8(value);
        
        key_ = value;
        onChanged();
        return this;
      }

      private java.lang.Object value_ = "";
      /**
       * <code>string value = 2;</code>
       * @return The value.
       */
      public java.lang.String getValue() {
        java.lang.Object ref = value_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          value_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>string value = 2;</code>
       * @return The bytes for value.
       */
      public com.google.protobuf.ByteString
          getValueBytes() {
        java.lang.Object ref = value_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          value_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>string value = 2;</code>
       * @param value The value to set.
       * @return This builder for chaining.
       */
      public Builder setValue(
          java.lang.String value) {
        if (value == null) {
    throw new NullPointerException();
  }
  
        value_ = value;
        onChanged();
        return this;
      }
      /**
       * <code>string value = 2;</code>
       * @return This builder for chaining.
       */
      public Builder clearValue() {
        
        value_ = getDefaultInstance().getValue();
        onChanged();
        return this;
      }
      /**
       * <code>string value = 2;</code>
       * @param value The bytes for value to set.
       * @return This builder for chaining.
       */
      public Builder setValueBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) {
    throw new NullPointerException();
  }
  checkByteStringIsUtf8(value);
        
        value_ = value;
        onChanged();
        return this;
      }
      @java.lang.Override
      public final Builder setUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return super.setUnknownFields(unknownFields);
      }

      @java.lang.Override
      public final Builder mergeUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return super.mergeUnknownFields(unknownFields);
      }


      // @@protoc_insertion_point(builder_scope:AttributeFieldEntry)
    }

    // @@protoc_insertion_point(class_scope:AttributeFieldEntry)
    private static final io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry();
    }

    public static io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<AttributeFieldEntry>
        PARSER = new com.google.protobuf.AbstractParser<AttributeFieldEntry>() {
      @java.lang.Override
      public AttributeFieldEntry parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        return new AttributeFieldEntry(input, extensionRegistry);
      }
    };

    public static com.google.protobuf.Parser<AttributeFieldEntry> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<AttributeFieldEntry> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public io.confluent.connect.protobuf.test.MapReferences.AttributeFieldEntry getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_MapReferencesMessage_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
      internal_static_MapReferencesMessage_fieldAccessorTable;
  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_AttributeFieldEntry_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
      internal_static_AttributeFieldEntry_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\023mapreferences.proto\"\256\001\n\024MapReferencesM" +
      "essage\022\"\n\004map1\030\001 \003(\0132\024.AttributeFieldEnt" +
      "ry\022\"\n\004map2\030\002 \003(\0132\024.AttributeFieldEntry\022&" +
      "\n\010notAMap1\030\003 \001(\0132\024.AttributeFieldEntry\022&" +
      "\n\010notAMap2\030\004 \001(\0132\024.AttributeFieldEntry\"1" +
      "\n\023AttributeFieldEntry\022\013\n\003key\030\001 \001(\t\022\r\n\005va" +
      "lue\030\002 \001(\tB3\n\"io.confluent.connect.protob" +
      "uf.testB\rMapReferencesb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
        });
    internal_static_MapReferencesMessage_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_MapReferencesMessage_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessageV3.FieldAccessorTable(
        internal_static_MapReferencesMessage_descriptor,
        new java.lang.String[] { "Map1", "Map2", "NotAMap1", "NotAMap2", });
    internal_static_AttributeFieldEntry_descriptor =
      getDescriptor().getMessageTypes().get(1);
    internal_static_AttributeFieldEntry_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessageV3.FieldAccessorTable(
        internal_static_AttributeFieldEntry_descriptor,
        new java.lang.String[] { "Key", "Value", });
  }

  // @@protoc_insertion_point(outer_class_scope)
}
